﻿using Un4seen.Bass;

namespace WpfAudioPlayer.Models.Audio
{
	/// <summary>
	/// 音声のコーラス効果を提供します。
	/// </summary>
	class EffectChorus : IEffect
	{
		/// <summary>
		/// インスタンスを初期化します。
		/// </summary>
		/// <param name="stream">ストリームの識別子。</param>
		public EffectChorus( int stream )
		{
			this._effect = new BASS_DX8_CHORUS( EffectChorus.DefaultWetDryMix, EffectChorus.DefaultDepth, EffectChorus.DefaultFeedback, 5f, 1, EffectChorus.DefaultDelay, BASSFXPhase.BASS_FX_PHASE_NEG_90 );
			this._stream   = stream;
		}

		/// <summary>
		/// 音響効果を更新します。
		/// </summary>
		private void UpdateEffect()
		{
			if( this.IsEnabled )
			{
				Bass.BASS_FXSetParameters( this._handle, this._effect );
			}
		}

		#region 定数

		/// <summary>
		/// 音声の遅延時間の既定値。
		/// </summary>
		public const float DefaultDelay = 0f;

		/// <summary>
		/// 変調の割合の既定値。
		/// </summary>
		public const float DefaultDepth = 25f;

		/// <summary>
		/// 効果音をフィードバックさせる比率の既定値。
		/// </summary>
		public const float DefaultFeedback = 90f;

		/// <summary>
		/// 効果音 ( Wet ) と原音 ( Dry ) の配分の既定値。
		/// </summary>
		public const float DefaultWetDryMix = 0f;

		#endregion

		#region プロパティ

		/// <summary>
		/// 音声の遅延時間を取得または設定します。
		/// </summary>
		public float Delay
		{
			get
			{
				return this._effect.fDelay;
			}
			set
			{
				this._effect.fDelay = value;
				this.UpdateEffect();
			}
		}

		/// <summary>
		/// 変調の割合を取得または設定します。
		/// </summary>
		public float Depth
		{
			get
			{
				return this._effect.fDepth;
			}
			set
			{
				this._effect.fDepth = value;
				this.UpdateEffect();
			}
		}
		
		/// <summary>
		/// 効果音をフィードバックさせる比率を取得または設定します。
		/// </summary>
		public float Feedback
		{
			get
			{
				return this._effect.fFeedback;
			}
			set
			{
				this._effect.fFeedback = value;
				this.UpdateEffect();
			}
		}

		/// <summary>
		/// 効果音 ( Wet ) と原音 ( Dry ) の配分を取得または設定します。
		/// </summary>
		public float WetDryMix
		{
			get
			{
				return this._effect.fWetDryMix;
			}
			set
			{
				this._effect.fWetDryMix = value;
				this.UpdateEffect();
			}
		}

		#endregion

		#region フィールド

		/// <summary>
		/// 音響効果。
		/// </summary>
		private BASS_DX8_CHORUS _effect;

		/// <summary>
		/// 音響効果の識別子。
		/// </summary>
		private int _handle;

		/// <summary>
		/// ストリームの識別子。
		/// </summary>
		private int _stream;

		#endregion

		#region IEffect メンバ

		/// <summary>
		/// 音響効果が有効である事を示す値を取得または設定します。
		/// </summary>
		public bool IsEnabled
		{
			get
			{
				return ( this._handle != 0 );
			}
			set
			{
				if( this._handle == 0 )
				{
					if( value )
					{
						this._handle = Bass.BASS_ChannelSetFX( this._stream, BASSFXType.BASS_FX_DX8_CHORUS, 1 );
						Bass.BASS_FXSetParameters( this._handle, this._effect );
					}
				}
				else if( !value )
				{
					Bass.BASS_ChannelRemoveFX( this._stream, this._handle );
					this._handle = 0;
				}
			}
		}

		#endregion
	}
}
