﻿using Un4seen.Bass;

namespace WpfAudioPlayer.Models.Audio
{
	/// <summary>
	/// 音声のディストーション効果を提供します。
	/// </summary>
	class EffectDistortion : IEffect
	{
		/// <summary>
		/// インスタンスを初期化します。
		/// </summary>
		/// <param name="stream">ストリームの識別子。</param>
		public EffectDistortion( int stream )
		{
			this._effect = new BASS_DX8_DISTORTION();
			this._stream = stream;
		}
	
		/// <summary>
		/// 音響効果を更新します。
		/// </summary>
		private void UpdateEffect()
		{
			if( this.IsEnabled )
			{
				Bass.BASS_FXSetParameters( this._handle, this._effect );
			}
		}

		#region プロパティ

		/// <summary>
		/// 歪みの強さを取得または設定します。
		/// </summary>
		public float Edge
		{
			get
			{
				return this._effect.fEdge;
			}
			set
			{
				this._effect.fEdge = value;
				this.UpdateEffect();
			}
		}

		/// <summary>
		/// 歪みを掛けた時の信号の変化量を取得または設定します。
		/// </summary>
		public float Gain
		{
			get
			{
				return this._effect.fGain;
			}
			set
			{
				this._effect.fGain = value;
				this.UpdateEffect();
			}
		}

		/// <summary>
		/// 高調波成分の範囲を取得または設定します。
		/// </summary>
		public float PostEqBandWidth
		{
			get
			{
				return this._effect.fPostEQBandwidth;
			}
			set
			{
				this._effect.fPostEQBandwidth = value;
				this.UpdateEffect();
			}
		}

		/// <summary>
		/// 中心周波数の範囲を取得または設定します。
		/// </summary>
		public float PostEqCenterFrequency
		{
			get
			{
				return this._effect.fPostEQCenterFrequency;
			}
			set
			{
				this._effect.fPostEQCenterFrequency = value;
				this.UpdateEffect();
			}
		}

		/// <summary>
		/// 減衰フィルタの範囲を取得または設定します。
		/// </summary>
		public float PreLowPassCutOff
		{
			get
			{
				return this._effect.fPreLowpassCutoff;
			}
			set
			{
				this._effect.fPreLowpassCutoff = value;
				this.UpdateEffect();
			}
		}

		#endregion

		#region 定数

		/// <summary>
		/// 歪みの強さの既定値。
		/// </summary>
		public const float DefaultEdge = 50f;

		/// <summary>
		/// 歪みを掛けた時の信号の変化量の既定値。
		/// </summary>
		public const float DefaultGain = 0f;

		/// <summary>
		/// 高調波成分の範囲の既定値。
		/// </summary>
		public const float DefaultPostEqBandWidth = 4000f;

		/// <summary>
		/// 中心周波数の範囲の既定値。
		/// </summary>
		public const float DefaultPostEqCenterFrequency = 4000f;

		/// <summary>
		/// 減衰フィルタの範囲の既定値。
		/// </summary>
		public const float DefaultPreLowPassCutOff = 4000f;

		#endregion

		#region フィールド

		/// <summary>
		/// 音響効果。
		/// </summary>
		private BASS_DX8_DISTORTION _effect;
		
		/// <summary>
		/// 音響効果の識別子。
		/// </summary>
		private int _handle;

		/// <summary>
		/// ストリームの識別子。
		/// </summary>
		private int _stream;

		#endregion

		#region IEffect メンバ

		/// <summary>
		/// 音響効果が有効である事を示す値を取得または設定します。
		/// </summary>
		public bool IsEnabled
		{
			get
			{
				return ( this._handle != 0 );
			}
			set
			{
				if( this._handle == 0 )
				{
					if( value )
					{
						this._handle = Bass.BASS_ChannelSetFX( this._stream, BASSFXType.BASS_FX_BFX_DISTORTION, 3 );
						Bass.BASS_FXSetParameters( this._handle, this._effect );
					}
				}
				else if( !value )
				{
					Bass.BASS_ChannelRemoveFX( this._stream, this._handle );
					this._handle = 0;
				}
			}
		}

		#endregion
	}
}
