﻿using Un4seen.Bass;

namespace WpfAudioPlayer.Models.Audio
{
	/// <summary>
	/// 音声のエコー効果を提供します。
	/// </summary>
	class EffectEcho : IEffect
	{
		/// <summary>
		/// インスタンスを初期化します。
		/// </summary>
		/// <param name="stream">ストリームの識別子。</param>
		public EffectEcho( int stream )
		{
			this._effect = new BASS_DX8_ECHO( 90f, 50f, 500f, 500f, false );
			this._stream   = stream;
		}

		/// <summary>
		/// 音響効果を更新します。
		/// </summary>
		private void UpdateEffect()
		{
			if( this.IsEnabled )
			{
				Bass.BASS_FXSetParameters( this._handle, this._effect );
			}
		}

		#region プロパティ

		/// <summary>
		/// 効果音をフィードバックさせる比率を取得または設定します。
		/// </summary>
		public float Feedback
		{
			get
			{
				return this._effect.fFeedback;
			}
			set
			{
				this._effect.fFeedback = value;
				this.UpdateEffect();
			}
		}

		/// <summary>
		/// 左チャンネルの音声の遅延時間を取得または設定します。
		/// </summary>
		public float LeftDelay
		{
			get
			{
				return this._effect.fLeftDelay;
			}
			set
			{
				this._effect.fLeftDelay = value;
				this.UpdateEffect();
			}
		}

		/// <summary>
		/// 右チャンネルの音声の遅延時間を取得または設定します。
		/// </summary>
		public float RightDelay
		{
			get
			{
				return this._effect.fRightDelay;
			}
			set
			{
				this._effect.fRightDelay = value;
				this.UpdateEffect();
			}
		}

		/// <summary>
		/// 効果音 ( Wet ) と原音 ( Dry ) の配分を取得または設定します。
		/// </summary>
		public float WetDryMix
		{
			get
			{
				return this._effect.fWetDryMix;
			}
			set
			{
				this._effect.fWetDryMix = value;
				this.UpdateEffect();
			}
		}

		#endregion

		#region 定数

		/// <summary>
		/// 効果音をフィードバックさせる比率の既定値。
		/// </summary>
		public const float DefaultFeedback = 0f;

		/// <summary>
		/// 左チャンネルの音声の遅延時間の既定値。
		/// </summary>
		public const float DefaultLeftDelay = 333f;

		/// <summary>
		/// 右チャンネルの音声の遅延時間の既定値。
		/// </summary>
		public const float DefaultRightDelay = 333f;

		/// <summary>
		/// 効果音 ( Wet ) と原音 ( Dry ) の配分の既定値。
		/// </summary>
		public const float DefaultWetDryMix = 0f;

		#endregion

		#region フィールド

		/// <summary>
		/// 音響効果。
		/// </summary>
		private BASS_DX8_ECHO _effect;

		/// <summary>
		/// 音響効果の識別子。
		/// </summary>
		private int _handle;

		/// <summary>
		/// ストリームの識別子。
		/// </summary>
		private int _stream;

		#endregion

		#region IEffect メンバ

		/// <summary>
		/// 音響効果が有効である事を示す値を取得または設定します。
		/// </summary>
		public bool IsEnabled
		{
			get
			{
				return ( this._handle != 0 );
			}
			set
			{
				if( this._handle == 0 )
				{
					if( value )
					{
						this._handle = Bass.BASS_ChannelSetFX( this._stream, BASSFXType.BASS_FX_BFX_ECHO, 2 );
						Bass.BASS_FXSetParameters( this._handle, this._effect );
					}
				}
				else if( !value )
				{
					Bass.BASS_ChannelRemoveFX( this._stream, this._handle );
					this._handle = 0;
				}
			}
		}

		#endregion
	}
}
