﻿using System;
using System.IO;
using Un4seen.Bass;
using Un4seen.Bass.AddOn.Fx;

namespace WpfAudioPlayer.Models.Audio
{
	/// <summary>
	/// Bass.Net を利用して音楽再生を行います。
	/// </summary>
	sealed class Player : IDisposable
	{
		/// <summary>
		/// インスタンスを初期化します。
		/// </summary>
		/// <param name="fileName">音楽ファイルへのパス。</param>
		/// <exception cref="FileNotFoundException">音楽ファイルが存在しない。</exception>
		/// <exception cref="Exception">ストリームの生成に失敗した。</exception>
		public Player( string fileName )
		{
			if( !File.Exists( fileName ) ) { throw new FileNotFoundException( fileName ); }

			// ストリーム生成
			this._handle = Bass.BASS_StreamCreateFile( fileName, 0, 0, BASSFlag.BASS_SAMPLE_FLOAT );
			if( this._handle == 0 )
			{
				var error = Bass.BASS_ErrorGetCode();
				throw new Exception( "ストリームの生成に失敗しました。\nError : " + error.ToString() );
			}

			// 演奏時間の算出
			{
				var length  = Bass.BASS_ChannelGetLength( this._handle );
				var seconds = Bass.BASS_ChannelBytes2Seconds( this._handle, length );
				this.Duration = TimeSpan.FromSeconds( seconds );
			}

			this.PlayState = PlayState.Stopped;
		}

		/// <summary>
		/// Bass.Net を解放します。
		/// オーディオ再生が実行されている場合は停止されます。
		/// このメソッドを呼び出した場合、再度 BassInitialize メソッドを呼び出すまで Bass.Net は利用できません。
		/// </summary>
		public static void BassFree()
		{
			if( !Player.IsBassInitialized ) { return; }

			Bass.BASS_Stop();
			Bass.BASS_PluginFree( 0 );
			BassFx.FreeMe();
			Bass.FreeMe();

			Player.IsBassInitialized = false;
		}

		/// <summary>
		/// Bass.Net を初期化します。
		/// Bass.Net のライフタイムは、このメソッドから BassFree メソッドの呼び出しまでとなります。
		/// </summary>
		/// <param name="folderPath">BASS ライブラリのモジュールを格納したフォルダを示すパス文字列。</param>
		/// <exception cref="Exception">初期化に失敗した。</exception>
		public static void BassInitialize( string folderPath )
		{
			if( Player.IsBassInitialized ) { return; }

			// Bass.Net のスプラッシュ スクリーンを抑止
			//BassNet.Registration( "mail-address", "register-code" );

			// Bass.Net
			if( !Bass.LoadMe( folderPath ) )
			{
				throw new Exception( "Bass.Net の初期化に失敗しました。" );
			}

			// BassFx
			if( !BassFx.LoadMe( folderPath ) )
			{
				throw new Exception( "BassFx の初期化に失敗しました。" );
			}

			// デバイス
			if( !Bass.BASS_Init( -1, 44100, BASSInit.BASS_DEVICE_DEFAULT, IntPtr.Zero ) )
			{
				var error = Bass.BASS_ErrorGetCode();
				throw new Exception( "デバイスの初期化に失敗しました。\nError : " + error.ToString() );
			}

			// プラグイン
			{
				var plugins = Bass.BASS_PluginLoadDirectory( folderPath );
				Player.FileFilter = Utils.BASSAddOnGetPluginFileFilter( plugins, null );
			}

			Player.IsBassInitialized = true;
		}

		/// <summary>
		/// ファイル ダイアログに指定する為のフィルタ文字列を取得します。
		/// </summary>
		public static string FileFilter { get; private set; }

		/// <summary>
		/// Bass.Net の初期化を終えている事を示す値を取得または設定します。
		/// </summary>
		private static bool IsBassInitialized { get; set; }

		/// <summary>
		/// 再生を一時停止します。
		/// </summary>
		public void Pause()
		{
			if( this.PlayState == PlayState.Playing )
			{
				this.PlayState = PlayState.Paused;
				Bass.BASS_ChannelPause( this._handle );
			}
		}

		/// <summary>
		/// 再生を開始します。
		/// </summary>
		public void Play()
		{
			if( this.PlayState != PlayState.Playing )
			{
				this.PlayState = PlayState.Playing;
				Bass.BASS_ChannelPlay( this._handle, false );
			}
		}

		/// <summary>
		/// 再生を停止します。
		/// </summary>
		public void Stop()
		{
			if( this.PlayState != PlayState.Stopped )
			{
				this.PlayState = PlayState.Stopped;
				Bass.BASS_ChannelStop( this._handle );
				Bass.BASS_ChannelSetPosition( this._handle, 0.0 );
			}
		}

		/// <summary>
		/// 再生位置の変更を行える事を示す値を取得します。
		/// </summary>
		public bool CanSeek { get { return true; } }

		/// <summary>
		/// 再生位置を時間単位で取得または設定します。
		/// </summary>
		public TimeSpan CurrentTime
		{
			get
			{
				var position = Bass.BASS_ChannelGetPosition( this._handle );
				return TimeSpan.FromSeconds( Bass.BASS_ChannelBytes2Seconds( this._handle, position ) );
			}
			set
			{
				var position = Bass.BASS_ChannelSeconds2Bytes( this._handle, value.TotalSeconds );
				Bass.BASS_ChannelSetPosition( this._handle, position );
			}
		}

		/// <summary>
		/// 演奏時間を取得します。
		/// </summary>
		public TimeSpan Duration { get; private set; }

		/// <summary>
		/// 音楽再生の状態を取得します。
		/// </summary>
		public PlayState PlayState { get; private set; }

		/// <summary>
		/// ストリームのハンドルを取得します。
		/// </summary>
		public int Stream { get { return this._handle; } }

		/// <summary>
		/// 音量を取得または設定します。
		/// </summary>
		public float Volume
		{
			get
			{
				return this._volume;
			}
			set
			{
				this._volume = value;
				Bass.BASS_ChannelSetAttribute( this._handle, BASSAttribute.BASS_ATTRIB_VOL, value );
			}
		}

		#region IDisposable メンバ

		/// <summary>
		/// リソースの解放を行います。
		/// </summary>
		public void Dispose()
		{
			if( this._handle != 0 )
			{
				this.Stop();
				Bass.BASS_StreamFree( this._handle );
				this._handle = 0;
			}
		}

		#endregion

		#region フィールド

		/// <summary>
		/// オーディオ再生を行う為のストリームのハンドル。
		/// </summary>
		private int _handle;

		/// <summary>
		/// 音量。
		/// </summary>
		private float _volume = 1.0f;

		#endregion
	}

	/// <summary>
	/// 音楽再生の状態を表します。
	/// </summary>
	enum PlayState
	{
		/// <summary>
		/// 再生中。
		/// </summary>
		Playing,

		/// <summary>
		/// 一時停止。
		/// </summary>
		Paused,

		/// <summary>
		/// 停止。
		/// </summary>
		Stopped
	}

}
